interface _Headers {
    [key: string]: string;
}
type Diff<T, U> = T extends U ? {} : T;
type Opts<T> = Diff<RequestInit, {
    body?: BodyInit;
}> & {
    body?: T;
    headers?: _Headers;
};
type Method = 'ACL' | 'BIND' | 'CHECKOUT' | 'CONNECT' | 'COPY' | 'DELETE' | 'GET' | 'HEAD' | 'LINK' | 'LOCK' | 'M-SEARCH' | 'MERGE' | 'MKACTIVITY' | 'MKCALENDAR' | 'MKCOL' | 'MOVE' | 'NOTIFY' | 'OPTIONS' | 'PATCH' | 'POST' | 'PROPFIND' | 'PROPPATCH' | 'PURGE' | 'PUT' | 'REBIND' | 'REPORT' | 'SEARCH' | 'SOURCE' | 'SUBSCRIBE' | 'TRACE' | 'UNBIND' | 'UNLINK' | 'UNLOCK' | 'UNSUBSCRIBE';
export declare const trim: (s: string) => string;
export declare const eq: <T>(expected: T) => ((a: T) => boolean);
export declare const hasJsonBody: (res: Response) => boolean;
export interface SimpleResponse<T> {
    ok: boolean;
    status: number;
    headers: Headers;
    body: T;
}
export declare const toSimpleResponse: <T>(res: Response, body: T) => SimpleResponse<T>;
export declare const getBodyOrFail: <T>(res: SimpleResponse<T>) => Promise<T>;
/**
 * Performs an ajax call with headers and
 * includes full response object. If given, the request
 * body will be JSON stringified and any response body
 * will be parsed as JSON.
 */
export declare const sendJsonR: <ReqT extends {}, ResT>(method: Method, url: string, options?: Opts<ReqT>) => Promise<SimpleResponse<ResT | void>>;
/**
 * Performs an ajax call with headers, returning
 * only the response body. If given, the request
 * body will be JSON stringified and any response body
 * will be parsed as JSON.
 */
export declare const sendJson: <ReqT extends {}, ResT>(method: Method, url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export declare const del: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export declare const get: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export declare const head: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<SimpleResponse<ResT | void>>;
export declare const patch: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export declare const post: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export declare const put: <ReqT extends {}, ResT>(url: string, options?: Opts<ReqT>) => Promise<ResT | void>;
export {};
